// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::SocketAddressEnumerator;
use glib::object::IsA;
use glib::translate::*;
use std::fmt;

glib::wrapper! {
    #[doc(alias = "GSocketConnectable")]
    pub struct SocketConnectable(Interface<ffi::GSocketConnectable, ffi::GSocketConnectableIface>);

    match fn {
        type_ => || ffi::g_socket_connectable_get_type(),
    }
}

pub const NONE_SOCKET_CONNECTABLE: Option<&SocketConnectable> = None;

pub trait SocketConnectableExt: 'static {
    #[doc(alias = "g_socket_connectable_enumerate")]
    fn enumerate(&self) -> SocketAddressEnumerator;

    #[doc(alias = "g_socket_connectable_proxy_enumerate")]
    fn proxy_enumerate(&self) -> SocketAddressEnumerator;

    #[doc(alias = "g_socket_connectable_to_string")]
    fn to_string(&self) -> glib::GString;
}

impl<O: IsA<SocketConnectable>> SocketConnectableExt for O {
    fn enumerate(&self) -> SocketAddressEnumerator {
        unsafe {
            from_glib_full(ffi::g_socket_connectable_enumerate(
                self.as_ref().to_glib_none().0,
            ))
        }
    }

    fn proxy_enumerate(&self) -> SocketAddressEnumerator {
        unsafe {
            from_glib_full(ffi::g_socket_connectable_proxy_enumerate(
                self.as_ref().to_glib_none().0,
            ))
        }
    }

    fn to_string(&self) -> glib::GString {
        unsafe {
            from_glib_full(ffi::g_socket_connectable_to_string(
                self.as_ref().to_glib_none().0,
            ))
        }
    }
}

impl fmt::Display for SocketConnectable {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        f.write_str("SocketConnectable")
    }
}
