// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::SocketAddress;
use crate::SocketConnectable;
use crate::UnixSocketAddressType;
use glib::object::IsA;
use glib::translate::*;
use glib::StaticType;
use std::fmt;

glib::wrapper! {
    #[doc(alias = "GUnixSocketAddress")]
    pub struct UnixSocketAddress(Object<ffi::GUnixSocketAddress, ffi::GUnixSocketAddressClass>) @extends SocketAddress, @implements SocketConnectable;

    match fn {
        type_ => || ffi::g_unix_socket_address_get_type(),
    }
}

impl UnixSocketAddress {
    //#[doc(alias = "g_unix_socket_address_new_abstract")]
    //pub fn new_abstract(path: /*Unimplemented*/&CArray TypeId { ns_id: 0, id: 10 }) -> UnixSocketAddress {
    //    unsafe { TODO: call ffi:g_unix_socket_address_new_abstract() }
    //}

    //#[doc(alias = "g_unix_socket_address_new_with_type")]
    //#[doc(alias = "new_with_type")]
    //pub fn with_type(path: /*Unimplemented*/&CArray TypeId { ns_id: 0, id: 10 }, type_: UnixSocketAddressType) -> UnixSocketAddress {
    //    unsafe { TODO: call ffi:g_unix_socket_address_new_with_type() }
    //}

    #[doc(alias = "g_unix_socket_address_abstract_names_supported")]
    pub fn abstract_names_supported() -> bool {
        unsafe { from_glib(ffi::g_unix_socket_address_abstract_names_supported()) }
    }
}

unsafe impl Send for UnixSocketAddress {}
unsafe impl Sync for UnixSocketAddress {}

pub const NONE_UNIX_SOCKET_ADDRESS: Option<&UnixSocketAddress> = None;

pub trait UnixSocketAddressExt: 'static {
    #[doc(alias = "g_unix_socket_address_get_address_type")]
    #[doc(alias = "get_address_type")]
    fn address_type(&self) -> UnixSocketAddressType;

    #[doc(alias = "g_unix_socket_address_get_is_abstract")]
    #[doc(alias = "get_is_abstract")]
    fn is_abstract(&self) -> bool;

    #[doc(alias = "g_unix_socket_address_get_path_len")]
    #[doc(alias = "get_path_len")]
    fn path_len(&self) -> usize;

    #[doc(alias = "path-as-array")]
    fn path_as_array(&self) -> Option<glib::ByteArray>;
}

impl<O: IsA<UnixSocketAddress>> UnixSocketAddressExt for O {
    fn address_type(&self) -> UnixSocketAddressType {
        unsafe {
            from_glib(ffi::g_unix_socket_address_get_address_type(
                self.as_ref().to_glib_none().0,
            ))
        }
    }

    fn is_abstract(&self) -> bool {
        unsafe {
            from_glib(ffi::g_unix_socket_address_get_is_abstract(
                self.as_ref().to_glib_none().0,
            ))
        }
    }

    fn path_len(&self) -> usize {
        unsafe { ffi::g_unix_socket_address_get_path_len(self.as_ref().to_glib_none().0) }
    }

    fn path_as_array(&self) -> Option<glib::ByteArray> {
        unsafe {
            let mut value = glib::Value::from_type(<glib::ByteArray as StaticType>::static_type());
            glib::gobject_ffi::g_object_get_property(
                self.to_glib_none().0 as *mut glib::gobject_ffi::GObject,
                b"path-as-array\0".as_ptr() as *const _,
                value.to_glib_none_mut().0,
            );
            value
                .get()
                .expect("Return Value for property `path-as-array` getter")
        }
    }
}

impl fmt::Display for UnixSocketAddress {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        f.write_str("UnixSocketAddress")
    }
}
